// Copyright 2020 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Code generated by client-gen. DO NOT EDIT.

package v1alpha1

import (
	"net/http"

	v1alpha1 "github.com/google/kf/v2/pkg/apis/kf/v1alpha1"
	"github.com/google/kf/v2/pkg/client/kf/clientset/versioned/scheme"
	rest "k8s.io/client-go/rest"
)

type KfV1alpha1Interface interface {
	RESTClient() rest.Interface
	AppsGetter
	BuildsGetter
	ClusterServiceBrokersGetter
	RoutesGetter
	ScalesGetter
	ServiceBrokersGetter
	ServiceInstancesGetter
	ServiceInstanceBindingsGetter
	SourcePackagesGetter
	SpacesGetter
	TasksGetter
	TaskSchedulesGetter
}

// KfV1alpha1Client is used to interact with features provided by the kf.dev group.
type KfV1alpha1Client struct {
	restClient rest.Interface
}

func (c *KfV1alpha1Client) Apps(namespace string) AppInterface {
	return newApps(c, namespace)
}

func (c *KfV1alpha1Client) Builds(namespace string) BuildInterface {
	return newBuilds(c, namespace)
}

func (c *KfV1alpha1Client) ClusterServiceBrokers() ClusterServiceBrokerInterface {
	return newClusterServiceBrokers(c)
}

func (c *KfV1alpha1Client) Routes(namespace string) RouteInterface {
	return newRoutes(c, namespace)
}

func (c *KfV1alpha1Client) Scales(namespace string) ScaleInterface {
	return newScales(c, namespace)
}

func (c *KfV1alpha1Client) ServiceBrokers(namespace string) ServiceBrokerInterface {
	return newServiceBrokers(c, namespace)
}

func (c *KfV1alpha1Client) ServiceInstances(namespace string) ServiceInstanceInterface {
	return newServiceInstances(c, namespace)
}

func (c *KfV1alpha1Client) ServiceInstanceBindings(namespace string) ServiceInstanceBindingInterface {
	return newServiceInstanceBindings(c, namespace)
}

func (c *KfV1alpha1Client) SourcePackages(namespace string) SourcePackageInterface {
	return newSourcePackages(c, namespace)
}

func (c *KfV1alpha1Client) Spaces() SpaceInterface {
	return newSpaces(c)
}

func (c *KfV1alpha1Client) Tasks(namespace string) TaskInterface {
	return newTasks(c, namespace)
}

func (c *KfV1alpha1Client) TaskSchedules(namespace string) TaskScheduleInterface {
	return newTaskSchedules(c, namespace)
}

// NewForConfig creates a new KfV1alpha1Client for the given config.
// NewForConfig is equivalent to NewForConfigAndClient(c, httpClient),
// where httpClient was generated with rest.HTTPClientFor(c).
func NewForConfig(c *rest.Config) (*KfV1alpha1Client, error) {
	config := *c
	if err := setConfigDefaults(&config); err != nil {
		return nil, err
	}
	httpClient, err := rest.HTTPClientFor(&config)
	if err != nil {
		return nil, err
	}
	return NewForConfigAndClient(&config, httpClient)
}

// NewForConfigAndClient creates a new KfV1alpha1Client for the given config and http client.
// Note the http client provided takes precedence over the configured transport values.
func NewForConfigAndClient(c *rest.Config, h *http.Client) (*KfV1alpha1Client, error) {
	config := *c
	if err := setConfigDefaults(&config); err != nil {
		return nil, err
	}
	client, err := rest.RESTClientForConfigAndClient(&config, h)
	if err != nil {
		return nil, err
	}
	return &KfV1alpha1Client{client}, nil
}

// NewForConfigOrDie creates a new KfV1alpha1Client for the given config and
// panics if there is an error in the config.
func NewForConfigOrDie(c *rest.Config) *KfV1alpha1Client {
	client, err := NewForConfig(c)
	if err != nil {
		panic(err)
	}
	return client
}

// New creates a new KfV1alpha1Client for the given RESTClient.
func New(c rest.Interface) *KfV1alpha1Client {
	return &KfV1alpha1Client{c}
}

func setConfigDefaults(config *rest.Config) error {
	gv := v1alpha1.SchemeGroupVersion
	config.GroupVersion = &gv
	config.APIPath = "/apis"
	config.NegotiatedSerializer = scheme.Codecs.WithoutConversion()

	if config.UserAgent == "" {
		config.UserAgent = rest.DefaultKubernetesUserAgent()
	}

	return nil
}

// RESTClient returns a RESTClient that is used to communicate
// with API server by this client implementation.
func (c *KfV1alpha1Client) RESTClient() rest.Interface {
	if c == nil {
		return nil
	}
	return c.restClient
}
